package de.hamster.debugger.model;

import de.hamster.model.*;

/**
 * Dies ist die Oberklasse eines imperativen Hamsterprogramms. Ein imperatives
 * Hamsterprogramm steuert immer den Standardhamster und unterstuetzt dabei die
 * Befehle vor, linksUm, nimm, gib, vornFrei, maulLeer und kornDa. Ausserdem
 * kann ein imperatives Hamsterprogramm jetzt auch ueber die Methoden schreib,
 * liesZahl und liesZeichenkette eine gewisse Ein- und Ausgabe ermoeglichen.
 * 
 * @author $Author: djasper $
 * @version $Revision: 1.2 $
 */
public class IHamster extends Thread {
	/**
	 * Der InstructionProcessor
	 */
	static InstructionProcessor processor;

	/**
	 * Ein imperativer Hamster hat immer die ID -1
	 */
	int id = -1;

	public void vor() {
		Object o = processor.process(new HamsterInstruction(
				HamsterInstruction.FORWARD, id));
		if (o instanceof MauerDaException) {
			MauerDaException e = (MauerDaException) o;
			throw new MauerDaException(null, e.getReihe(), e.getSpalte());
		}
	}

	public void linksUm() {
		processor.process(new HamsterInstruction(HamsterInstruction.TURN_LEFT,
				id));
	}

	public void nimm() {
		Object o = processor.process(new HamsterInstruction(
				HamsterInstruction.PICK_UP, id));
		if (o instanceof KachelLeerException) {
			KachelLeerException e = (KachelLeerException) o;
			throw new KachelLeerException(null, e.getReihe(), e.getSpalte());
		}
	}

	public void gib() {
		Object o = processor.process(new HamsterInstruction(
				HamsterInstruction.LAY_DOWN, id));
		if (o instanceof MaulLeerException)
			throw new MaulLeerException(null);
	}

	public boolean vornFrei() {
		Boolean b = (Boolean) processor.process(new HamsterInstruction(
				HamsterInstruction.FREE, id));
		return b.booleanValue();
	}

	public boolean maulLeer() {
		Boolean b = (Boolean) processor.process(new HamsterInstruction(
				HamsterInstruction.MOUTH_EMPTY, id));
		return b.booleanValue();
	}

	public boolean kornDa() {
		Boolean b = (Boolean) processor.process(new HamsterInstruction(
				HamsterInstruction.CORN_AVAILABLE, id));
		return b.booleanValue();
	}

	public void schreib(String text) {
		processor.process(new TerminalInstruction(TerminalInstruction.WRITE,
				id, text));
	}

	public int liesZahl(String message) {
		Integer i = (Integer) processor.process(new TerminalInstruction(
				TerminalInstruction.READ_INT, id, message));
		return i.intValue();
	}

	public String liesZeichenkette(String message) {
		return (String) processor.process(new TerminalInstruction(
				TerminalInstruction.READ_STRING, id, message));
	}
}